from pyparsing import *

def delimList(rule, delim=","):
    return rule + ZeroOrMore(Suppress(Literal(delim)) + rule) | Empty ()

def props_to_dict(s,l,ts):
    dict = {}
    for t in ts:
        dict[t[0]]=t[1]
    return dict

prefix = Word ("^=*", exact = 1)
id = Word (alphas+"_-")
topic = id
event = Group(prefix + topic)
name = id
object = Forward()
array = Suppress("[") + Group(delimList (object)) + Suppress("]")
string = dblQuotedString.setParseAction(removeQuotes)
value = string | array | object
property = Group(name + Suppress("=") + value)
props = (delimList (property))
object << Suppress("{") + props.setParseAction(props_to_dict) + Suppress("}")
infoline = event + Suppress(",") + props



from __future__ import print_function
from pexpect import spawn
from pty import openpty
import threading
import os
from logging import info
import logging
import sys

logging.basicConfig(level=logging.DEBUG)

from parse_gdb_mi import infoline

class PrintStream(threading.Thread):

    def __init__(self, stream_descriptor, lineHandler=print):
        self.file = os.fdopen(stream_descriptor)
        threading.Thread.__init__(self)
        self.lineHandler = lineHandler

    def run(self):
        info("PrintStream started")
        while True:
            line = self.file.readline()
            if not line: break
            info ("line from stream: %s " % line)
            self.lineHandler (line)
        info("PrintStream exited")

class ControlledExec(object):

    def wait_prompt(self):
        self.debug_console.expect ("\(gdb\)")

    def __init__(self, exec_path, logfile = None, out_stream_handler = print):
        self.exec_path = exec_path
        self.out_stream_handler = out_stream_handler
        # allocate a pty for controlling
        self.master, self.slave = openpty()
        exec_string = "gdb %s -tty %s -interpreter mi" % (self.exec_path,os.ttyname(self.slave))
        info("starting %s" % exec_string)
        self.debug_console = spawn(exec_string, logfile = logfile)
        self.wait_prompt()

    def run_stream_handler(self):
        self.print_stream_handler = PrintStream(self.master,lineHandler = self.out_stream_handler)
        self.print_stream_handler.daemon = True
        self.print_stream_handler.start()

    def run_command(self, cmd):
        outstring = ""
        outstring += self.console_command(cmd)
        self.wait_prompt()


    def get_parsed_info(self,cmd):
        out = self.console_cmd(cmd)
        info = out.splitlines()[2]
        return infoline.parseString(info).asList()[1]

    def thread_info(self):
        return self.get_parsed_info("-thread-info")


    def get_variables(self):
        return self.get_parsed_info("-stack-list-variables 2")

    def get_file_info(self):
        return self.get_parsed_info("-file-list-exec-source-file")

    def exec_run(self):
        self.console_cmd("-exec-run")
        self.wait_prompt()

    def close(self):
        self.debug_console.close()


    def exec_continue(self):
        self.console_cmd("-exec-continue")
        self.wait_prompt()

    def next(self):
        self.console_cmd("-exec-next")
        self.wait_prompt()

    def console_cmd(self, cmd):
        self.debug_console.sendline(cmd)
        self.wait_prompt()
        return self.debug_console.before

    def set_break(self, point):
        return self.console_cmd("-break-insert %s" % point)

    def run_inferior(self):
        child = self.debug_console
        self.run_stream_handler()
        child.expect("\(gdb\)")
        child.sendline("-exec-run")
        child.expect(".*exited.*")
        child.sendline("quit")
        child.readlines()




import os
from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "pyexeccontrol",
    version = "0.0.1",
    author = "Francesco Bruschi",
    author_email = "fbrusch@gmail.com",
    description = "An interface to gdb to control program execution",
    license = "BSD",
    keywords = "gdb debug execution",
    url = "http://packages.python.org/pyexeccontrol",
    packages = ["execcontrol"],
    long_description = read('README'),
    classifiers = [
        "Development Status :: 2 - Pre-Alpha"]
)
